REM
REM  ZilogBoy - A GameBoy emulator for Windows, Linux and DOS
REM  Copyright (C) Mateusz Viste 2010
REM

'#DEFINE DEBUGMODE
#DEFINE PrintDebug REM

TYPE RomInformations
  Title AS STRING
  ColorGB AS BYTE  ' 0 = not color, 1 = color
  SuperGB AS BYTE  ' 0 = GameBoy, 1 = Super GameBoy
  CartType AS STRING
  Destination AS BYTE  ' 0 = Japan, 1 = Other
  Licensee AS STRING
  RomSize AS INTEGER
  RamSize AS INTEGER
END TYPE

DIM SHARED RomInfos AS RomInformations
DIM SHARED CartTypes(0 TO 255) AS STRING
DIM SHARED AS BYTE QuitEmulator
DIM SHARED AS UBYTE GraphicScaleFactor = 4
DIM SHARED AS UINTEGER GraphicWidth, GraphicHeight

#INCLUDE ONCE "loadcarttypes.bi"  ' LoadCartTypes()
#INCLUDE ONCE "numconv.bi"        ' UbyteToByte() and DwordVal()
#INCLUDE ONCE "mmu.bi"            ' Include Memory Management Unit emulation core
#INCLUDE ONCE "video.bi"          ' Video emulation core
#INCLUDE ONCE "adjtiming.bi"      ' AdjustTiming()
#IFDEF DEBUGMODE
  #INCLUDE ONCE "debug.bi"          ' Includes some debugging routines
#ENDIF
#INCLUDE ONCE "cpu-z80.bi"        ' Z80 CPU emulation core
#INCLUDE ONCE "file.bi"           ' Provides the FileExists() function
#INCLUDE ONCE "loadrom.bi"        ' LoadROM(RomFile AS STRING)
#INCLUDE ONCE "loadrominfos.bi"   ' LoadRomInfos()
#INCLUDE ONCE "serial.bi"         ' Serial interface emulation
#INCLUDE ONCE "joypad.bi"         ' Joypad interface emulation
#INCLUDE ONCE "savepcxfile.bi"    ' Screenshot routine SavePcxFile()
#INCLUDE ONCE "loadzboypicture.bi"  ' LoadZboyPicture()

DIM NoYes(0 TO 1) AS STRING => {"No", "Yes"}
DIM AS ULONGINT NextKeybPoll, NextCpuRelease
DIM LastKey AS STRING

#IFDEF DEBUGMODE
  GraphicWidth = 800
  GraphicHeight = 900
  GraphicScaleFactor = 1
#ELSE
  GraphicWidth = 160 * GraphicScaleFactor
  GraphicHeight = 144 * GraphicScaleFactor
#ENDIF

#IFDEF __FB_WIN32__  ' Use the GDI driver on Windows instead of DirectX (for compatibility reasons)
  SCREENCONTROL(fb.SET_DRIVER_NAME, "GDI")
#ENDIF
SCREENRES GraphicWidth, GraphicHeight, 32, 1     ' init graphic window with 32bpp and 1 video page

LoadCartTypes()
IF LEN(COMMAND(1)) = 0 THEN
  PRINT "Please specify a ROM file to load!"
  LoadZboyPicture()
  RefreshScreen()
  SLEEP
  WHILE LEN(INKEY) > 0 : WEND
  END
END IF

IF LEFT(COMMAND(1), 2) = "--" THEN
    PRINT "Got some unknown switch."
    END
  ELSE
    LoadROM(COMMAND(1))
END IF

LoadRomInfos()
PRINT "Title......: "; RomInfos.Title
PRINT "Licensee...: "; RomInfos.Licensee
PRINT "ColorGB....: "; NoYes(RomInfos.ColorGB)
PRINT "SuperGB....: "; NoYes(RomInfos.SuperGB)
PRINT "Cart Type..: "; RomInfos.CartType
IF RomInfos.Destination = 0 THEN
    PRINT "Country....: Japan"
  ELSE
    PRINT "Country....: Other (not Japan)"
END IF
PRINT "ROM Size...: " & (RomInfos.RomSize \ 1024) & " KiB"
PRINT "RAM Size...: " & (RomInfos.RamSize \ 1024) & " KiB"

IF RomInfos.SuperGB = 1 THEN PRINT : PRINT "Error: SuperGB ROMs are not supported (yet). Sorry." : END
IF RomInfos.ColorGB = 1 THEN PRINT : PRINT "Error: GameBoy Color ROMs are not supported (yet). Sorry." : END
IF RomInfos.RomSize > 32768 THEN PRINT : PRINT "Error: ROMs bigger than 32K aren't supported (yet). Sorry." : END
IF RomInfos.RamSize > 0 THEN PRINT : PRINT "Error: ROMs with internal RAM aren't supported (yet). Sorry." : END

#IFDEF DEBUGMODE
  COLOR RGB(200,200,200)
  InitDebug()
#ENDIF

RANDOMIZE TIMER, 3  ' Use Mersenne Twister RND algorithm

'InitRAM()    ' Init RAM areas to random values (as in a real GameBoy)
CpuPowerUp() ' Fire up the Zilog80 monster
DO
  CpuExec()
  CheckSerialLink()  ' Check serial link (not supported yet)
  CheckJoypad()      ' Updates the Joypad register
  CheckInterrupts()  ' Check if any interrupt has been requested
  #IFDEF DEBUGMODE
    DebugSpriteDump(1)  ' perform some debug dumps onscreen (OBJ table at &h8000)
    DebugSpriteDump(2)  ' perform some debug dumps onscreen (OBJ table at &h8800)
    DebugPaletteDump()  ' Dump system's palette onscreen
    DebugShowCPUspeed() ' Calc current CPU speed and put it onscreen
  #ENDIF
  'IF TotalCycles > NextCpuRelease THEN
  '  NextCpuRelease += 1000000   ' Release a CPU cycle every 250ms
  '  SLEEP 1,1   ' Release a CPU cycle
  'END IF
  IF TotalCycles > NextKeybPoll THEN
    NextKeybPoll += 2000000  ' Poll the keyboard buffer every 500 ms.
    LastKey = INKEY
    SELECT CASE LastKey     ' Escape
      CASE CHR(27)
        QuitEmulator = 1
      CASE CHR(255) + "k"   ' Close window signal
        QuitEmulator = 2
      CASE CHR(255,67)      ' if F9 has been pressed...
        SavePcxFile("zboy.pcx", 160, 144)
    END SELECT
  END IF
LOOP UNTIL QuitEmulator > 0

#IFDEF DEBUGMODE
  IF QuitEmulator = 1 THEN
    LOCATE ,50: PRINT "--------------------------------"
    SLEEP
  END IF
#ENDIF

WHILE LEN(INKEY) > 0 : WEND  ' Flush keyb buffer
END
